@@ -30,12 +30,21 @@ opt_group = cfg.OptGroup(name='xenapi',
 
 OPTS = [
     cfg.StrOpt('connection_url',
+               deprecated_for_removal=True,
+               deprecated_reason='Support for XenServer/Xen Cloud Platform '
+                                 'has been deprecated',
                help='URL for connection to XenServer/Xen Cloud Platform.'),
     cfg.StrOpt('connection_username',
                default='root',
+               deprecated_for_removal=True,
+               deprecated_reason='Support for XenServer/Xen Cloud Platform '
+                                 'has been deprecated',
                help='Username for connection to XenServer/Xen Cloud '
                     'Platform.'),
     cfg.StrOpt('connection_password',
+               deprecated_for_removal=True,
+               deprecated_reason='Support for XenServer/Xen Cloud Platform '
+                                 'has been deprecated',
                help='Password for connection to XenServer/Xen Cloud Platform.',
                secret=True),
 ]
@@ -127,7 +136,7 @@ class XenapiInspector(virt_inspector.Inspector):
         LOG.debug("inspect_vnics, all bandwidth: %s", bw_all,
                   instance=instance)
 
-        for vif_ref in vif_refs or []:
+        for vif_ref in vif_refs:
             vif_rec = self.session.VIF.get_record(vif_ref)
 
             bw_vif = bw_all[dom_id][vif_rec['device']]
@@ -141,44 +150,43 @@ class XenapiInspector(virt_inspector.Inspector):
                 parameters=None,
                 rx_bytes=bw_vif['bw_in'], rx_packets=-1, rx_drop=-1,
                 rx_errors=-1, tx_bytes=bw_vif['bw_out'], tx_packets=-1,
-                tx_drop=-1, tx_errors=-1)
+                tx_drop=-1, tx_errors=-1, rx_bytes_delta=-1,
+                tx_bytes_delta=-1)
 
     def inspect_vnic_rates(self, instance, duration):
         instance_name = util.instance_name(instance)
         vm_ref = self._lookup_by_name(instance_name)
         vif_refs = self.session.VM.get_VIFs(vm_ref)
-        if vif_refs:
-            for vif_ref in vif_refs:
-                vif_rec = self.session.VIF.get_record(vif_ref)
+        for vif_ref in vif_refs:
+            vif_rec = self.session.VIF.get_record(vif_ref)
 
-                rx_rate = float(self.session.VM.query_data_source(
-                    vm_ref, "vif_%s_rx" % vif_rec['device']))
-                tx_rate = float(self.session.VM.query_data_source(
-                    vm_ref, "vif_%s_tx" % vif_rec['device']))
+            rx_rate = float(self.session.VM.query_data_source(
+                vm_ref, "vif_%s_rx" % vif_rec['device']))
+            tx_rate = float(self.session.VM.query_data_source(
+                vm_ref, "vif_%s_tx" % vif_rec['device']))
 
-                yield virt_inspector.InterfaceRateStats(
-                    name=vif_rec['uuid'],
-                    mac=vif_rec['MAC'],
-                    fref=None,
-                    parameters=None,
-                    rx_bytes_rate=rx_rate,
-                    tx_bytes_rate=tx_rate)
+            yield virt_inspector.InterfaceRateStats(
+                name=vif_rec['uuid'],
+                mac=vif_rec['MAC'],
+                fref=None,
+                parameters=None,
+                rx_bytes_rate=rx_rate,
+                tx_bytes_rate=tx_rate)
 
     def inspect_disk_rates(self, instance, duration):
         instance_name = util.instance_name(instance)
         vm_ref = self._lookup_by_name(instance_name)
         vbd_refs = self.session.VM.get_VBDs(vm_ref)
-        if vbd_refs:
-            for vbd_ref in vbd_refs:
-                vbd_rec = self.session.VBD.get_record(vbd_ref)
+        for vbd_ref in vbd_refs:
+            vbd_rec = self.session.VBD.get_record(vbd_ref)
 
-                read_rate = float(self.session.VM.query_data_source(
-                    vm_ref, "vbd_%s_read" % vbd_rec['device']))
-                write_rate = float(self.session.VM.query_data_source(
-                    vm_ref, "vbd_%s_write" % vbd_rec['device']))
-                yield virt_inspector.DiskRateStats(
-                    device=vbd_rec['device'],
-                    read_bytes_rate=read_rate,
-                    read_requests_rate=0,
-                    write_bytes_rate=write_rate,
-                    write_requests_rate=0)
+            read_rate = float(self.session.VM.query_data_source(
+                vm_ref, "vbd_%s_read" % vbd_rec['device']))
+            write_rate = float(self.session.VM.query_data_source(
+                vm_ref, "vbd_%s_write" % vbd_rec['device']))
+            yield virt_inspector.DiskRateStats(
+                device=vbd_rec['device'],
+                read_bytes_rate=read_rate,
+                read_requests_rate=0,
+                write_bytes_rate=write_rate,
+                write_requests_rate=0)
