@@ -33,7 +33,7 @@ from functools import reduce
 import operator
 import requests
 
-from six.moves.urllib import parse as url_parse
+from urllib import parse as urlparse
 
 LOG = log.getLogger(__name__)
 
@@ -59,7 +59,7 @@ class PollsterDefinitionBuilder(object):
         if not supported_definitions:
             raise declarative.DynamicPollsterDefinitionException(
                 "Your configurations do not fit any type of DynamicPollsters, "
-                "please recheck them. Used configurations = [%s]." %
+                "please recheck them. Used configurations are [%s]." %
                 configurations)
 
         definition_name = self.join_supported_definitions_names(
@@ -111,10 +111,14 @@ class PollsterSampleExtractor(object):
     def generate_sample(self, pollster_sample, pollster_definitons=None):
         pollster_definitions =\
             pollster_definitons or self.definitions.configurations
-        metadata = []
+
+        metadata = dict()
         if 'metadata_fields' in pollster_definitions:
-            metadata = dict((k, pollster_sample.get(k))
-                            for k in pollster_definitions['metadata_fields'])
+            for k in pollster_definitions['metadata_fields']:
+                val = self.retrieve_attribute_nested_value(pollster_sample, k)
+
+                LOG.debug("Assigning value [%s] to metadata key [%s].", val, k)
+                metadata[k] = val
 
         self.generate_new_metadata_fields(
             metadata=metadata, pollster_definitions=pollster_definitions)
@@ -134,12 +138,20 @@ class PollsterSampleExtractor(object):
 
         attribute_key = value_attribute or self.definitions.\
             extract_attribute_key()
-        LOG.debug("Retrieving the nested keys [%s] from [%s].",
-                  attribute_key, json_object)
+
+        LOG.debug(
+            "Retrieving the nested keys [%s] from [%s] or pollster [""%s].",
+            attribute_key, json_object,
+            self.definitions.configurations["name"])
+
         keys_and_operations = attribute_key.split("|")
         attribute_key = keys_and_operations[0].strip()
-        nested_keys = attribute_key.split(".")
-        value = reduce(operator.getitem, nested_keys, json_object)
+
+        if attribute_key == ".":
+            value = json_object
+        else:
+            nested_keys = attribute_key.split(".")
+            value = reduce(operator.getitem, nested_keys, json_object)
 
         return self.operate_value(keys_and_operations, value)
 
@@ -154,11 +166,15 @@ class PollsterSampleExtractor(object):
                     "The attribute field operation [%s] must use the ["
                     "value] variable." % operation,
                     self.definitions.configurations)
-            LOG.debug("Executing operation [%s] against value [%s].",
-                      operation, value)
+
+            LOG.debug("Executing operation [%s] against value[%s] for "
+                      "pollster [%s].", operation, value,
+                      self.definitions.configurations["name"])
+
             value = eval(operation.strip())
-            LOG.debug("Result [%s] of operation [%s].",
-                      value, operation)
+            LOG.debug(
+                "Result [%s] of operation [%s] for pollster [%s].",
+                value, operation, self.definitions.configurations["name"])
         return value
 
 
@@ -315,10 +331,39 @@ class PollsterValueMapper(object):
 
 
 class PollsterDefinition(object):
+    """Represents a dynamic pollster configuration/parameter
+
+    It abstract the job of developers when creating or extending parameters,
+    such as validating parameters name, values and so on.
+    """
 
     def __init__(self, name, required=False, on_missing=lambda df: df.default,
                  default=None, validation_regex=None, creatable=True,
                  validator=None):
+        """Create a dynamic pollster configuration/parameter
+
+        :param name: the name of the pollster parameter/configuration.
+        :param required: indicates if the configuration/parameter is
+               optional or not.
+        :param on_missing: function that is executed when the
+               parameter/configuration is missing.
+        :param default: the default value to be used.
+        :param validation_regex: the regular expression used to validate the
+               name of the configuration/parameter.
+        :param creatable: it is an override mechanism to avoid creating
+               a configuration/parameter with the default value. The default
+               is ``True``; therefore, we always use the default value.
+               However, we can disable the use of the default value by
+               setting ``False``. When we set this configuration to
+               ``False``, the parameter is not added to the definition
+               dictionary if not defined by the operator in the pollster
+               YAML configuration file.
+        :param validator: function used to validate the value of the
+               parameter/configuration when it is given by the user. This
+               function signature should receive a value that is the value of
+               the parameter to be validate.
+        """
+
         self.name = name
         self.required = required
         self.on_missing = on_missing
@@ -361,7 +406,12 @@ class PollsterDefinitions(object):
         PollsterDefinition(name='default_value', default=-1),
         PollsterDefinition(name='metadata_mapping', default={}),
         PollsterDefinition(name='preserve_mapped_metadata', default=True),
-        PollsterDefinition(name='response_entries_key')]
+        PollsterDefinition(name='response_entries_key'),
+        PollsterDefinition(name='next_sample_url_attribute'),
+        PollsterDefinition(name='user_id_attribute', default="user_id"),
+        PollsterDefinition(name='resource_id_attribute', default="id"),
+        PollsterDefinition(name='project_id_attribute', default="project_id"),
+        PollsterDefinition(name='headers')]
 
     extra_definitions = []
 
@@ -481,19 +531,107 @@ class PollsterSampleGatherer(object):
                   response_json, url, self.definitions.configurations['name'])
 
         if entry_size > 0:
-            return self.retrieve_entries_from_response(response_json)
+            samples = self.retrieve_entries_from_response(response_json)
+            url_to_next_sample = self.get_url_to_next_sample(response_json)
+            if url_to_next_sample:
+                kwargs['next_sample_url'] = url_to_next_sample
+                samples += self.execute_request_get_samples(**kwargs)
+
+            self.execute_id_overrides(samples)
+            return samples
         return []
 
+    def execute_id_overrides(self, samples):
+        if samples:
+            user_id_attribute = self.definitions.configurations[
+                'user_id_attribute']
+            project_id_attribute = self.definitions.configurations[
+                'project_id_attribute']
+            resource_id_attribute = self.definitions.configurations[
+                'resource_id_attribute']
+
+            for request_sample in samples:
+                self.generate_new_attributes_in_sample(
+                    request_sample, user_id_attribute, 'user_id')
+                self.generate_new_attributes_in_sample(
+                    request_sample, project_id_attribute, 'project_id')
+                self.generate_new_attributes_in_sample(
+                    request_sample, resource_id_attribute, 'id')
+
+    def generate_new_attributes_in_sample(
+            self, sample, attribute_key, new_attribute_key):
+
+        if attribute_key == new_attribute_key:
+            LOG.debug("We do not need to generate new attribute as the "
+                      "attribute_key[%s] and the new_attribute_key[%s] "
+                      "configurations are the same.",
+                      attribute_key, new_attribute_key)
+            return
+
+        if attribute_key:
+            attribute_value = self.definitions.sample_extractor.\
+                retrieve_attribute_nested_value(sample, attribute_key)
+
+            LOG.debug("Mapped attribute [%s] to value [%s] in sample [%s].",
+                      attribute_key, attribute_value, sample)
+
+            sample[new_attribute_key] = attribute_value
+
+    def get_url_to_next_sample(self, resp):
+        linked_sample_extractor = self.definitions.configurations[
+            'next_sample_url_attribute']
+        if not linked_sample_extractor:
+            return None
+
+        try:
+            return self.definitions.sample_extractor.\
+                retrieve_attribute_nested_value(resp, linked_sample_extractor)
+        except KeyError:
+            LOG.debug("There is no next sample url for the sample [%s] using "
+                      "the configuration [%s]", resp, linked_sample_extractor)
+        return None
+
     def internal_execute_request_get_samples(self, kwargs):
         keystone_client = kwargs['keystone_client']
-        endpoint = kwargs['resource']
-        url = url_parse.urljoin(
-            endpoint, self.definitions.configurations['url_path'])
-        resp = keystone_client.session.get(url, authenticated=True)
+        url = self.get_request_linked_samples_url(kwargs)
+
+        request_arguments = self.create_request_arguments()
+        LOG.debug("Executing request against [url=%s] with parameters ["
+                  "%s] for pollsters [%s]", url, request_arguments,
+                  self.definitions.configurations["name"])
+
+        resp = keystone_client.session.get(url, **request_arguments)
+
         if resp.status_code != requests.codes.ok:
             resp.raise_for_status()
         return resp, url
 
+    def create_request_arguments(self):
+        request_args = {
+            "authenticated": True
+        }
+        request_headers = self.definitions.configurations['headers']
+        if request_headers:
+            request_args['headers'] = request_headers
+        return request_args
+
+    def get_request_linked_samples_url(self, kwargs):
+        next_sample_url = kwargs.get('next_sample_url')
+        if next_sample_url:
+            return self.get_next_page_url(kwargs, next_sample_url)
+        return self.get_request_url(
+            kwargs, self.definitions.configurations['url_path'])
+
+    def get_next_page_url(self, kwargs, next_sample_url):
+        parse_result = urlparse.urlparse(next_sample_url)
+        if parse_result.netloc:
+            return next_sample_url
+        return self.get_request_url(kwargs, next_sample_url)
+
+    def get_request_url(self, kwargs, url_path):
+        endpoint = kwargs['resource']
+        return urlparse.urljoin(endpoint, url_path)
+
     def retrieve_entries_from_response(self, response_json):
         if isinstance(response_json, list):
             return response_json
@@ -516,11 +654,8 @@ class NonOpenStackApisPollsterDefinition(PollsterDefinitions):
         PollsterDefinition(name='value_attribute', required=True),
         PollsterDefinition(name='module', required=True),
         PollsterDefinition(name='authentication_object', required=True),
-        PollsterDefinition(name='user_id_attribute'),
-        PollsterDefinition(name='resource_id_attribute'),
         PollsterDefinition(name='barbican_secret_id', default=""),
         PollsterDefinition(name='authentication_parameters', default=""),
-        PollsterDefinition(name='project_id_attribute'),
         PollsterDefinition(name='endpoint_type')]
 
     def __init__(self, configurations):
@@ -548,7 +683,7 @@ class NonOpenStackApisSamplesGatherer(PollsterSampleGatherer):
         if override_credentials:
             credentials = override_credentials
 
-        url = self.definitions.configurations['url_path']
+        url = self.get_request_linked_samples_url(kwargs)
 
         authenticator_module_name = self.definitions.configurations['module']
         authenticator_class_name = \
@@ -561,9 +696,13 @@ class NonOpenStackApisSamplesGatherer(PollsterSampleGatherer):
         authenticator_arguments = list(map(str.strip, credentials.split(",")))
         authenticator_instance = authenticator_class(*authenticator_arguments)
 
-        resp = requests.get(
-            url,
-            auth=authenticator_instance)
+        request_arguments = self.create_request_arguments()
+        request_arguments["auth"] = authenticator_instance
+
+        LOG.debug("Executing request against [url=%s] with parameters ["
+                  "%s] for pollsters [%s]", url, request_arguments,
+                  self.definitions.configurations["name"])
+        resp = requests.get(url, **request_arguments)
 
         if resp.status_code != requests.codes.ok:
             raise declarative.NonOpenStackApisDynamicPollsterException(
@@ -573,6 +712,20 @@ class NonOpenStackApisSamplesGatherer(PollsterSampleGatherer):
 
         return resp, url
 
+    def create_request_arguments(self):
+        request_arguments = super(
+            NonOpenStackApisSamplesGatherer, self).create_request_arguments()
+
+        request_arguments.pop("authenticated")
+
+        return request_arguments
+
+    def get_request_url(self, kwargs, url_path):
+        endpoint = self.definitions.configurations['url_path']
+        if endpoint == url_path:
+            return url_path
+        return urlparse.urljoin(endpoint, url_path)
+
     def execute_request_get_samples(self, **kwargs):
         samples = super(NonOpenStackApisSamplesGatherer,
                         self).execute_request_get_samples(**kwargs)
@@ -636,7 +789,7 @@ class DynamicPollster(plugin_base.PollsterBase):
 
     @property
     def default_discovery(self):
-        return self.definitions.sample_gatherer.default_discovery()
+        return self.definitions.sample_gatherer.default_discovery
 
     def load_samples(self, resource, manager):
         try:
