@@ -26,9 +26,8 @@ from keystoneauth1 import exceptions as ka_exceptions
 from oslo_log import log
 from oslo_utils import fnmatch
 from oslo_utils import timeutils
-import six
-import six.moves.urllib.parse as urlparse
 from stevedore import extension
+from urllib import parse as urlparse
 
 from ceilometer import declarative
 from ceilometer import gnocchi_client
@@ -43,8 +42,7 @@ LOG = log.getLogger(__name__)
 
 def cache_key_mangler(key):
     """Construct an opaque cache key."""
-    if six.PY2:
-        key = key.encode('utf-8')
+
     return uuid.uuid5(CACHE_NAMESPACE, key).hex
 
 
@@ -53,10 +51,10 @@ EVENT_CREATE, EVENT_UPDATE, EVENT_DELETE = ("create", "update", "delete")
 
 class ResourcesDefinition(object):
 
-    MANDATORY_FIELDS = {'resource_type': six.string_types,
+    MANDATORY_FIELDS = {'resource_type': str,
                         'metrics': (dict, list)}
 
-    MANDATORY_EVENT_FIELDS = {'id': six.string_types}
+    MANDATORY_EVENT_FIELDS = {'id': str}
 
     def __init__(self, definition_cfg, archive_policy_default,
                  archive_policy_override, plugin_manager):
@@ -336,11 +334,10 @@ class GnocchiPublisher(publisher.ConfigPublisherBase):
         gnocchi_data = {}
         measures = {}
         for resource_id, samples_of_resource in resource_grouped_samples:
-            # NOTE(sileht): / is forbidden by Gnocchi
-            resource_id = resource_id.replace('/', '_')
-
             for sample in samples_of_resource:
                 metric_name = sample.name
+                LOG.debug("Processing sample [%s] for resource ID [%s].",
+                          sample, resource_id)
                 rd = self.metric_map.get(metric_name)
                 if rd is None:
                     if metric_name not in self._already_logged_metric_names:
@@ -349,6 +346,8 @@ class GnocchiPublisher(publisher.ConfigPublisherBase):
                         self._already_logged_metric_names.add(metric_name)
                     continue
 
+                # NOTE(sileht): / is forbidden by Gnocchi
+                resource_id = resource_id.replace('/', '_')
                 if resource_id not in gnocchi_data:
                     gnocchi_data[resource_id] = {
                         'resource_type': rd.cfg['resource_type'],
@@ -372,9 +371,9 @@ class GnocchiPublisher(publisher.ConfigPublisherBase):
         try:
             self.batch_measures(measures, gnocchi_data)
         except gnocchi_exc.ClientException as e:
-            LOG.error(six.text_type(e))
+            LOG.error(str(e))
         except Exception as e:
-            LOG.error(six.text_type(e), exc_info=True)
+            LOG.error(str(e), exc_info=True)
 
         for info in gnocchi_data.values():
             resource = info["resource"]
@@ -386,9 +385,9 @@ class GnocchiPublisher(publisher.ConfigPublisherBase):
                 self._if_not_cached(resource_type, resource['id'],
                                     resource_extra)
             except gnocchi_exc.ClientException as e:
-                LOG.error(six.text_type(e))
+                LOG.error(str(e))
             except Exception as e:
-                LOG.error(six.text_type(e), exc_info=True)
+                LOG.error(str(e), exc_info=True)
 
     @staticmethod
     def _extract_resources_from_error(e, resource_infos):
@@ -421,7 +420,7 @@ class GnocchiPublisher(publisher.ConfigPublisherBase):
                     pass
                 except gnocchi_exc.ClientException as e:
                     LOG.error('Error creating resource %(id)s: %(err)s',
-                              {'id': resource['id'], 'err': six.text_type(e)})
+                              {'id': resource['id'], 'err': str(e)})
                     # We cannot post measures for this resource
                     # and we can't patch it later
                     del measures[resource['id']]
